use crate::fan::hysteresis::HysteresisConfig;
use crate::fan::pwm::PwmConfig;

use serde::{Deserialize, Serialize};
use std::time::Duration;

#[derive(Debug, Serialize, Deserialize)]
pub struct Config {
    pub check_interval: Duration,
    pub mode: FanMode,
    pub verbosity_mode: VerbosityMode,
}

#[derive(Debug, Serialize, Deserialize)]
pub enum FanMode {
    Hysteresis(HysteresisConfig),
    Pwm(PwmConfig),
}

#[derive(Debug, Serialize, Deserialize)]
pub enum VerbosityMode {
    Human,
    Machine,
    Quiet,
}

impl Default for Config {
    fn default() -> Self {
        Config {
            check_interval: Duration::from_secs(10),
            mode: FanMode::Hysteresis(HysteresisConfig::default()),
            //mode: FanMode::Pwm(PwmConfig::default()),
            verbosity_mode: VerbosityMode::Human,
        }
    }
}
