use crate::fan::Fan;

use rppal::gpio::{Gpio, OutputPin};
use serde::{Deserialize, Serialize};

#[derive(Debug, Serialize, Deserialize)]
pub struct HysteresisConfig {
    max_temperature: f64,
    hysteresis: f64,
    bcm_pin: u8,
}

impl Default for HysteresisConfig {
    fn default() -> Self {
        HysteresisConfig {
            max_temperature: 70_f64,
            hysteresis: 10_f64,
            bcm_pin: 18,
        }
    }
}

pub struct Hysteresis {
    max_temperature: f64,
    hysteresis: f64,
    pin: OutputPin,
}

impl Hysteresis {
    pub fn new(config: HysteresisConfig) -> Self {
        let gpio = Gpio::new().unwrap();
        let mut pin = gpio.get(config.bcm_pin).unwrap().into_output();

        pin.set_low();

        Hysteresis {
            max_temperature: config.max_temperature,
            hysteresis: config.hysteresis,
            pin,
        }
    }
}

impl Fan for Hysteresis {
    fn update_temperature(&mut self, temperature: f64) {
        if temperature > self.max_temperature {
            self.pin.set_high();
        } else if temperature < self.max_temperature - self.hysteresis {
            self.pin.set_low();
        }
    }

    fn percentage(&self) -> f64 {
        if self.pin.is_set_high() {
            1_f64
        } else {
            0_f64
        }
    }
}
