mod config;
mod fan;
mod temperatures;

use config::{Config, FanMode::*, VerbosityMode::*};
use fan::hysteresis::Hysteresis;
use fan::pwm::Pwm;
use fan::Fan;
use temperatures::Temperatures;

use std::error::Error;
use std::thread;

fn main() -> Result<(), Box<dyn Error>> {
    let mut temps = Temperatures::new()?;
    let config: Config = confy::load("tempi").unwrap();

    // Can be used to check configuration and/or save default one.
    //confy::store("tempi", dbg!(&config)).unwrap();

    let mut fan: Box<dyn Fan> = match config.mode {
        Hysteresis(h) => Box::new(Hysteresis::new(h)),
        Pwm(p) => Box::new(Pwm::new(p)),
    };

    loop {
        temps.update();

        match temps.max() {
            Some(t) => {
                fan.update_temperature(t);

                match config.verbosity_mode {
                    Human => println!(
                        "CPU: {:.1}°C, fan is at {:.1}%",
                        t,
                        fan.percentage() * 100_f64
                    ),
                    Machine => println!("{} | {}", t, fan.percentage()),
                    Quiet => (),
                }
            }
            None => {
                eprintln!("Recuring problem while getting the temperature");
            }
        }

        thread::sleep(config.check_interval);
    }
}
