use std::error::Error;
use std::fs::File;
use std::io::prelude::*;

pub struct Temperatures {
    cpu: f64,
    cpu_error_count: i32,
}

impl Temperatures {
    pub fn new() -> Result<Self, Box<dyn Error>> {
        Ok(Temperatures {
            cpu: cpu()?,
            cpu_error_count: 0,
        })
    }

    pub fn update(&mut self) {
        match cpu() {
            Ok(t) => {
                self.cpu = t;
                self.cpu_error_count = 0;
            }
            Err(_) => {
                self.cpu_error_count += 1;
            }
        }
    }

    pub fn max(&self) -> Option<f64> {
        if self.cpu_error_count < 5 {
            Some(self.cpu)
        } else {
            None
        }
    }
}

fn cpu() -> Result<f64, Box<dyn Error>> {
    let mut f = File::open("/sys/class/thermal/thermal_zone0/temp")?;
    let mut buffer = String::new();
    f.read_to_string(&mut buffer)?;

    Ok(buffer.trim().parse::<f64>()? / 1_000_f64)
}
